"""
:py:mod:`tests.base`

Base test cases for *all* tests.


Copyright (C) 2016-2017 Mathias Stelzer

xtract is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

xtract is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
from __future__ import absolute_import, print_function, unicode_literals

import os
import shutil
import tempfile
import unittest


THIS_FILE = os.path.abspath(__file__)
THIS_DIR = os.path.dirname(THIS_FILE)

TEST_FILES_DIR = os.path.join(THIS_DIR, 'test_files')


class BaseTestCase(unittest.TestCase):
    def setUp(self):
        self.tmp_root = os.path.join(tempfile.gettempdir(), 'xtract_tests')

        self.tmp_dir = os.path.join(self.tmp_root, 'tmp_dir')
        shutil.rmtree(self.tmp_root, ignore_errors=True)
        os.makedirs(self.tmp_dir)

    def tearDown(self):
        shutil.rmtree(self.tmp_root, ignore_errors=True)

    def require(self, relpath):
        source = os.path.join(TEST_FILES_DIR, relpath)
        destination = os.path.join(self.tmp_dir, os.path.basename(relpath))
        if os.path.isdir(source):
            shutil.rmtree(destination, ignore_errors=True)
            shutil.copytree(source, destination)
        else:
            shutil.copyfile(source, destination)
        return destination

    def call(self, *args, **kwargs):
        raise NotImplementedError


class AbstractFileTests(object):
    def test_file(self):
        raise NotImplementedError

    def test_file_destination(self):
        raise NotImplementedError

    def test_file_destination_directory(self):
        raise NotImplementedError

    def test_file_destination_exists(self):
        raise NotImplementedError


class AbstractDirectoryTests(object):
    def test_directory(self):
        raise NotImplementedError

    def test_directory_destination(self):
        raise NotImplementedError

    def test_directory_destination_directory(self):
        raise NotImplementedError


class AbstractXtractFileTests(AbstractFileTests):
    def test_file_mimetype(self):
        raise NotImplementedError

    def test_file_mimetype_destination(self):
        raise NotImplementedError

    def test_file_mimetype_destination_directory(self):
        raise NotImplementedError


class BaseXtractFileTests(AbstractXtractFileTests):
    def setUp(self):
        super(BaseXtractFileTests, self).setUp()

        self.file = os.path.join(self.tmp_dir, 'file.html')
        self.file_archive = self.require('file.html' + self.typ().default_extension)

    def test_file(self):
        return self.call(self.file_archive)

    def test_file_destination(self):
        self.destination = os.path.join(self.tmp_dir, 'other_name.html')

        return self.call(self.file_archive, self.destination)

    def test_file_destination_directory(self):
        self.destination = os.path.join(self.tmp_dir, 'file.html')

        return self.call(self.file_archive, self.tmp_dir)

    def test_file_destination_exists(self):
        self.destination = self.require('file.html')

        return self.call(self.file_archive, self.destination)

    def test_file_mimetype(self):
        self.source = os.path.join(self.tmp_dir, 'seems_to_be.compressed')
        shutil.move(self.file_archive, self.source)
        self.destination = os.path.join(self.tmp_dir, 'seems_to_be.compressed-xtract')

        return self.call(self.source, None)

    def test_file_mimetype_destination(self):
        self.source = os.path.join(self.tmp_dir, 'seems_to_be.compressed')
        shutil.move(self.file_archive, self.source)
        self.destination = os.path.join(self.tmp_dir, 'new_file.html')

        return self.call(self.source, self.destination)

    def test_file_mimetype_destination_directory(self):
        self.source = os.path.join(self.tmp_dir, 'seems_to_be.compressed')
        shutil.move(self.file_archive, self.source)
        self.destination = os.path.join(self.tmp_dir, 'seems_to_be.compressed-xtract')

        return self.call(self.source, self.tmp_dir)


class BaseXtractDirectoryTests(AbstractDirectoryTests):
    def setUp(self):
        super(BaseXtractDirectoryTests, self).setUp()

        self.dir = os.path.join(self.tmp_dir, 'dir')
        self.dir_archive = self.require('dir' + self.typ().default_extension)

    def test_directory(self):
        return self.call(self.dir_archive)

    def test_directory_destination(self):
        self.destination = os.path.join(self.tmp_dir, 'other_destination')

        return self.call(self.dir_archive, self.destination)

    def test_directory_destination_directory(self):
        return self.call(self.dir_archive, self.tmp_dir)


class BaseInsertFileTests(AbstractFileTests):
    def setUp(self):
        super(BaseInsertFileTests, self).setUp()

        self.file = self.require('file.html')
        self.file_archive = os.path.join(self.tmp_dir, 'file.html' + self.typ().default_extension)

    def test_file(self):
        return self.call(self.file)

    def test_file_destination(self):
        self.destination = os.path.join(self.tmp_dir, 'other_name.html.archive')

        return self.call(self.file, destination=self.destination)

    def test_file_destination_directory(self):
        self.destination = os.path.join(self.tmp_dir, 'file.html' + self.typ().default_extension)

        return self.call(self.file, destination=self.tmp_dir)

    def test_file_destination_exists(self):
        self.file_archive = self.require(os.path.basename(self.file_archive))

        return self.call(self.file, self.file_archive)


class BaseInsertDirectoryTests(AbstractDirectoryTests):
    def setUp(self):
        super(BaseInsertDirectoryTests, self).setUp()

        self.dir = self.require('dir')
        self.dir_archive = os.path.join(self.tmp_dir, 'dir' + self.typ().default_extension)

    def test_directory(self):
        return self.call(self.dir)

    def test_directory_destination(self):
        self.destination = os.path.join(self.tmp_dir, 'other_destination')

        return self.call(self.dir, destination=self.destination)

    def test_directory_destination_directory(self):
        return self.call(self.dir, destination=self.tmp_dir)

    def test_directory_source_files_single(self):
        return self.call(self.dir, ['file-a.txt'])

    def test_directory_source_files_multiple(self):
        return self.call(
            self.dir,
            [
                'file-a.txt',
                os.path.join('subdir', 'file-b.txt')
            ]
        )
