"""
:py:mod:`tests.functional.test_xtract`

Tests for the :py:mod:`xtract.xtract` module.


Copyright (C) 2016-2017 Mathias Stelzer

xtract is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

xtract is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
from __future__ import absolute_import, print_function, unicode_literals

import os
import sys
import unittest

from distutils.spawn import find_executable
from base import (
    BaseTestCase,
    BaseXtractDirectoryTests,
    BaseXtractFileTests,
    BaseInsertFileTests,
    BaseInsertDirectoryTests,
)
from xtract import xtract, Rar, Zip, Tar, GZip, XZ, BZip2, FileTypeNotSupported


if sys.version_info[0] < 3:
    FileExistsError = OSError


###############################################################################
# xtract


class BaseXtractTestCase(BaseTestCase):
    def call(self, *args, **kwargs):
        return xtract(*args, **kwargs)


class XtractFileTests(BaseXtractFileTests):
    def test_file(self):
        result = super(XtractFileTests, self).test_file()
        self.assertTrue(os.path.exists(result))

    def test_file_destination(self):
        result = super(XtractFileTests, self).test_file_destination()
        self.assertTrue(os.path.exists(result))

    def test_file_destination_directory(self):
        result = super(XtractFileTests, self).test_file_destination_directory()
        self.assertTrue(os.path.exists(result))

    def test_file_destination_exists(self):
        with self.assertRaises(FileExistsError):
            super(XtractFileTests, self).test_file_destination_exists()

    def test_file_mimetype(self):
        result = super(XtractFileTests, self).test_file_mimetype()
        self.assertTrue(os.path.exists(result))

    def test_file_mimetype_destination(self):
        result = super(XtractFileTests, self).test_file_mimetype_destination()
        self.assertTrue(os.path.exists(result))

    def test_file_mimetype_destination_directory(self):
        result = super(XtractFileTests, self).test_file_mimetype_destination_directory()
        self.assertTrue(os.path.exists(result))

    def test_file_type_not_supported(self):
        source = self.require('file.html')

        with self.assertRaises(FileTypeNotSupported):
            self.call(source)

    def test_file_all(self):
        os.unlink(self.file_archive)
        source = self.require('file.html{}'.format(self.typ().default_extension))

        result = self.call(source, all=True)

        self.assertEqual(self.file, result)
        self.assertTrue(os.path.exists(result))

    def test_all_file_type_not_supported(self):
        source = self.require('file.html')

        with self.assertRaises(FileTypeNotSupported):
            self.call(source, all=True)

    def test_file_overwrite(self):
        # store original content
        destination = self.require('file.html')
        with open(destination) as f:
            content = f.read()

        # change content
        with open(destination, 'w') as f:
            f.write('test_file_overwrite_' + self.typ().name)

        result = self.call(self.file_archive, overwrite=True)

        self.assertEqual(destination, result)

        # check its original content again
        with open(destination) as f:
            self.assertEqual(content, f.read())


class XtractDirectoryTests(BaseXtractDirectoryTests):
    def assert_dir_is_unpacked(self, at_path):
        self.assertTrue(os.path.exists(at_path))
        self.assertTrue(os.path.exists(os.path.join(at_path, 'dir/')))
        self.assertTrue(os.path.exists(os.path.join(at_path, 'dir/file-a.txt')))
        self.assertTrue(os.path.exists(os.path.join(at_path, 'dir/subdir/')))
        self.assertTrue(os.path.exists(os.path.join(at_path, 'dir/subdir/file-b.txt')))

    def test_directory(self):
        result = super(XtractDirectoryTests, self).test_directory()
        self.assert_dir_is_unpacked(result)

    def test_directory_destination(self):
        result = super(XtractDirectoryTests, self).test_directory_destination()
        self.assert_dir_is_unpacked(result)

    def test_directory_destination_directory(self):
        result = super(XtractDirectoryTests, self).test_directory_destination_directory()
        self.assert_dir_is_unpacked(result)

    def test_directory_all(self):
        os.unlink(self.dir_archive)
        source = self.require('dir{}.gz'.format(self.typ().default_extension))

        result = self.call(source, all=True)

        self.assertEqual(os.path.join(self.tmp_dir, 'dir'), result)
        self.assert_dir_is_unpacked(result)

    def test_file_overwrite(self):  # overwrite test from XtractFileTests
        destination = self.require('file.html')

        result = self.call(self.file_archive, overwrite=True)

        self.assertEqual(destination, result)
        self.assertTrue(os.path.isdir(destination))


@unittest.skipUnless(find_executable('rar'), 'rar is not installed')
@unittest.skipUnless(find_executable('unrar'), 'unrar is not installed')
class XtractRarTestCase(XtractDirectoryTests, XtractFileTests, BaseXtractTestCase):
    typ = Rar


@unittest.skipUnless(find_executable('zip'), 'zip is not installed')
@unittest.skipUnless(find_executable('unzip'), 'unzip is not installed')
class XtractZipTestCase(XtractDirectoryTests, XtractFileTests, BaseXtractTestCase):
    typ = Zip


@unittest.skipUnless(find_executable('tar'), 'tar is not installed')
class XtractTarTestCase(XtractDirectoryTests, XtractFileTests, BaseXtractTestCase):
    typ = Tar


@unittest.skipUnless(find_executable('gzip'), 'gzip is not installed')
class XtractGzipTestCase(XtractFileTests, BaseXtractTestCase):
    typ = GZip


@unittest.skipUnless(find_executable('xz'), 'xz is not installed')
class XtractXZTestCase(XtractFileTests, BaseXtractTestCase):
    typ = XZ


@unittest.skipUnless(find_executable('bzip2'), 'bzip2 is not installed')
class XtractBzip2TestCase(XtractFileTests, BaseXtractTestCase):
    typ = BZip2


###############################################################################
# pack / compress


class BasePackTestCase(BaseTestCase):
    def call(self, *args, **kwargs):
        return self.typ().pack(*args, **kwargs)


class BaseCompressTestCase(BaseTestCase):
    def call(self, *args, **kwargs):
        return self.typ().compress(*args, **kwargs)


class InsertFileTests(BaseInsertFileTests):
    def test_file(self):
        result = super(InsertFileTests, self).test_file()
        self.assertEqual(self.file_archive, result)
        self.assertTrue(os.path.exists(result))

    def test_file_destination(self):
        result = super(InsertFileTests, self).test_file_destination()
        self.assertEqual(self.destination + self.typ().default_extension, result)
        self.assertTrue(os.path.exists(result))

    def test_file_destination_directory(self):
        result = super(InsertFileTests, self).test_file_destination_directory()
        self.assertEqual(self.file_archive, result)
        self.assertTrue(os.path.exists(result))

    def test_file_destination_exists(self):
        with self.assertRaises(FileExistsError):
            super(InsertFileTests, self).test_file_destination_exists()


class InsertDirectoryTests(BaseInsertDirectoryTests):
    def test_directory(self):
        result = super(InsertDirectoryTests, self).test_directory()
        self.assertEqual(self.dir_archive, result)
        self.assertTrue(os.path.exists(result))

    def test_directory_destination(self):
        result = super(InsertDirectoryTests, self).test_directory_destination()
        self.assertEqual(self.destination + self.typ().default_extension, result)
        self.assertTrue(os.path.exists(result))

    def test_directory_destination_directory(self):
        result = super(InsertDirectoryTests, self).test_directory_destination_directory()
        self.assertEqual(self.dir_archive, result)
        self.assertTrue(os.path.exists(result))

    def test_directory_source_files_single(self):
        result = super(InsertDirectoryTests, self).test_directory_source_files_single()
        self.assertEqual(self.dir_archive, result)
        self.assertTrue(os.path.exists(result))

    def test_directory_source_files_multiple(self):
        result = super(InsertDirectoryTests, self).test_directory_source_files_multiple()
        self.assertEqual(self.dir_archive, result)
        self.assertTrue(os.path.exists(result))


@unittest.skipUnless(find_executable('rar'), 'rar is not installed')
@unittest.skipUnless(find_executable('unrar'), 'unrar is not installed')
class PackRarTestCase(InsertDirectoryTests, InsertFileTests, BasePackTestCase):
    typ = Rar


@unittest.skipUnless(find_executable('zip'), 'zip is not installed')
@unittest.skipUnless(find_executable('unzip'), 'unzip is not installed')
class PackZipTestCase(InsertDirectoryTests, InsertFileTests, BasePackTestCase):
    typ = Zip


@unittest.skipUnless(find_executable('tar'), 'tar is not installed')
class PackTarTestCase(InsertDirectoryTests, InsertFileTests, BasePackTestCase):
    typ = Tar


@unittest.skipUnless(find_executable('gzip'), 'gzip is not installed')
class CompressGZipTestCase(InsertFileTests, BaseCompressTestCase):
    typ = GZip


@unittest.skipUnless(find_executable('xz'), 'xz is not installed')
class CompressXZTestCase(InsertFileTests, BaseCompressTestCase):
    typ = XZ


@unittest.skipUnless(find_executable('bzip2'), 'bzip2 is not installed')
class CompressBZip2TestCase(InsertFileTests, BaseCompressTestCase):
    typ = BZip2

