#! /usr/bin/env python
"""
Test suite runner

Copyright (C) 2016-2017 Mathias Stelzer

xtract is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

xtract is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
from __future__ import absolute_import, print_function, unicode_literals

import os
import unittest
import shutil
import subprocess
import sys

from blessings import Terminal
from colour_runner.result import ColourTextTestResult, text_type
from unittest import TextTestRunner

try:
    import urllib.request
except ImportError:
    import urllib2

    def download(url, filename):
        response = urllib2.urlopen(url)
        with open(filename, 'wb') as out_file:
            shutil.copyfileobj(response, out_file)
else:
    def download(url, filename):
        with urllib.request.urlopen(url) as response, open(filename, 'wb') as out_file:
            shutil.copyfileobj(response, out_file)


__dirname__ = os.path.abspath(os.path.dirname(__file__))


TEST_FILES_URL = 'https://docs.rolln.de/knoppo/xtract/{}/test_files.tar.gz'
TEST_FILES_REMOTE = 'publisher@rolln.de:docs/knoppo/xtract/{}/test_files.tar.gz'
TEST_FILES_PATH = os.path.join(__dirname__, 'test_files')
TEST_FILES_ARCHIVE = os.path.join(__dirname__, 'test_files.tar.gz')


class MyColourTextTestResult(ColourTextTestResult):
    _terminal = Terminal(force_styling=True)
    colours = {
        None: text_type,
        'error': _terminal.red,
        'expected': _terminal.blue,
        'fail': _terminal.yellow,
        'skip': text_type,
        'success': _terminal.green,
        'title': _terminal.cyan,
        'unexpected': _terminal.red,
    }


class ColourTextTestRunner(TextTestRunner):
    resultclass = MyColourTextTestResult


def get_git_ref():
    cmd = ['git', 'rev-parse', '--abbrev-ref', 'HEAD']
    try:
        branch = subprocess.check_output(cmd, cwd=os.path.dirname(__dirname__)).decode().strip()
    except subprocess.CalledProcessError:
        branch = None
    if branch == 'HEAD' or branch is None:
        return 'master'
    return branch


def download_test_files(ref=None):
    if ref is None:
        ref = get_git_ref()

    if os.path.exists(TEST_FILES_PATH):
        shutil.rmtree(TEST_FILES_PATH)

    url = TEST_FILES_URL.format(ref)
    print('Downloading', url)
    download(url, TEST_FILES_ARCHIVE)

    dirname, basename = os.path.split(TEST_FILES_ARCHIVE)
    print('Unpacking', TEST_FILES_ARCHIVE)
    subprocess.call(['tar', 'xzvf', basename], cwd=dirname)

    sys.stdout.flush()

    if not os.path.exists(TEST_FILES_PATH):
        print('Failed to download test files for git ref "{}".'.format(ref))
        sys.exit(1)


def upload_test_files(ref=None):
    if ref is None:
        ref = get_git_ref()
    if not os.path.exists(TEST_FILES_PATH) or not os.listdir(TEST_FILES_PATH):
        print('ERROR: Nothing to upload')
        sys.exit(1)

    try:
        os.unlink(TEST_FILES_ARCHIVE)
    except OSError:
        pass

    dirname, basename = os.path.split(TEST_FILES_PATH)
    subprocess.check_call(['tar', 'czvf', TEST_FILES_ARCHIVE, basename], cwd=dirname)

    archive_dirname, archive_basename = os.path.split(TEST_FILES_ARCHIVE)
    remote = TEST_FILES_REMOTE.format(ref)
    subprocess.check_call(['scp', archive_basename, remote], cwd=archive_dirname)


def main(argv=None):
    if argv is None:
        argv = sys.argv[1:]

    if argv and argv[0] in ('upload', 'download'):
        ref = None
        if len(argv) > 1:
            ref = argv[1]
        if argv[0] == 'upload':
            upload_test_files(ref=ref)
        else:
            download_test_files(ref=ref)
        sys.exit(0)

    if not os.path.exists(TEST_FILES_PATH):
        download_test_files()

    suite = unittest.TestLoader().discover(__dirname__)
    result = ColourTextTestRunner(verbosity=2).run(suite)
    if not result.wasSuccessful():
        sys.exit(1)


if __name__ == '__main__':
    main()

